#!/usr/bin/env python
#
# Copyright 2016 The Fabulous Authors. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from fabulous.term import display, stdout, Magic

stdout.write(display('default'))
stdout.write("default text\n")

stdout.write("regular foreground test:\n")
for color in Magic.COLORS.iterkeys():
    stdout.write(display(fg=color))
    stdout.write("    " + color + '\n')

stdout.write(display('default'))

stdout.write("regular background test:\n")
for color in Magic.COLORS.iterkeys():
    stdout.write(display(bg=color))
    stdout.write("    " + color + '\n')

stdout.write(display('default'))

stdout.write("bright foreground test:\n")
stdout.write(display('bright'))
for color in Magic.COLORS.iterkeys():
    stdout.write(display(fg=color))
    stdout.write("    " + color + '\n')

stdout.write(display('default'))

stdout.write("dim foreground test:\n")
stdout.write(display('dim'))
for color in Magic.COLORS.iterkeys():
    stdout.write(display(fg=color))
    stdout.write("    " + color + '\n')

stdout.write(display('bright','red'))
stdout.write("bright red\n")
stdout.write(display("default"))
